@@ -155,8 +155,6 @@ class QueueMonitor:
                                                                                   self.activeJobs[jobId]['stage'], \
                                                                                   self.activeJobs[jobId]['queue']))
       self.historyFile.flush()
-#     self.updates.append(str(jobId) + ' ' + self.activeJobs[jobId][0] + ' ' + \
-#                                            self.activeJobs[jobId][1])
       jobState = {'jobId':jobId,
                   'status':self.activeJobs[jobId]['status'],
                   'stage':self.activeJobs[jobId]['stage'],
@@ -177,6 +175,7 @@ class QueueMonitor:
                   text = ""
             jobState['tailFiles'][tailFile] = text
 
+#     log(str(jobState))
       self.updates.append(jobState)
 
 
@@ -300,23 +299,25 @@ class QueueMonitor:
                   nLines,
                   lastEndPosition=0):
       text        = ""
+      nTextLines  = 0
       endPosition = lastEndPosition
       bufsize = 2048
       if os.path.exists(tailPath):
          fileSize = os.stat(tailPath).st_size
-         if lastEndPosition+1 <= fileSize:
+         if fileSize > lastEndPosition:
             try:
                fpTail = open(tailPath,'r')
                try:
                   iter = 0
                   while True:
                      iter += 1
-                     location = max(fileSize-bufsize*iter,lastEndPosition+1)
+                     location = max(fileSize-bufsize*iter,lastEndPosition)
                      fpTail.seek(location)
                      data = fpTail.readlines()
                      endPosition = fpTail.tell()
-                     if len(data) > nLines or location == lastEndPosition+1:
+                     if len(data) > nLines or location == lastEndPosition:
                         text = ''.join(data[-nLines:])
+                        nTextLines += min(len(data),nLines)
                         if lastEndPosition > 0 and location > lastEndPosition+1:
                            text = "...\n" + text
                         del data
@@ -328,6 +329,9 @@ class QueueMonitor:
             except (IOError,OSError):
                log("%s could not be opened" % (tailPath))
 
+      if nTextLines > 0:
+         log("tailed %d lines of %s" % (nTextLines,tailPath))
+
       return(text,endPosition)
 
 
@@ -355,51 +359,52 @@ class QueueMonitor:
 
             ready = select.select(toCheck,[],[],self.pauseTime) # wait for input
             if sys.stdin.fileno() in ready[0]:
-#              newJob = sys.stdin.readline().strip()
                newJob = ""
                message = sys.stdin.readline()
+#              log(message)
+
                if message != '':
                   try:
                      centralMessage = json.loads(message)
                   except:
                      log(traceback.format_exc())
                   else:
+#                    log("READ: " + str(centralMessage))
                      if centralMessage['messageType'] == 'newJobId':
                         newJob = centralMessage['remoteJobId']
                   if newJob != "":
-                     if not newJob in self.activeJobs:
-                        self.activeJobs[newJob] = {'jobId':newJob,
-                                                   'status':'N',
-                                                   'stage':'Job',
-                                                   'queue':'?',
-                                                   'jobWorkDirectory':centralMessage.get('jobWorkDirectory','?'),
-                                                   'localJobId':centralMessage.get('localJobId','?'),
-                                                   'instanceId':centralMessage.get('instanceId','?'),
-                                                   'runName':centralMessage.get('runName','?'),
-                                                   'tailFiles':centralMessage.get('tailFiles',[])}
-                        jobWorkDirectory = os.path.expandvars(os.path.expanduser(self.activeJobs[newJob]['jobWorkDirectory']))
-                        if 'tailFiles' in self.activeJobs[newJob]:
-                           if 'runName' in self.activeJobs[newJob]:
-                              runName = self.activeJobs[newJob]['runName']
-                              if 'instanceId' in self.activeJobs[newJob]:
-                                 instanceId = self.activeJobs[newJob]['instanceId']
-                                 if "#STDOUT#" in self.activeJobs[newJob]['tailFiles']:
-                                    stdFile = "%s_%s.stdout" % (runName,instanceId)
-                                    self.activeJobs[newJob]['tailFiles']["#STDOUT#"]['path'] = os.path.join(jobWorkDirectory,
-                                                                                                            stdFile)
-                                    self.activeJobs[newJob]['tailFiles']["#STDOUT#"]['endPosition'] = 0
-                                 if "#STDERR#" in self.activeJobs[newJob]['tailFiles']:
-                                    stdFile = "%s_%s.stderr" % (runName,instanceId)
-                                    self.activeJobs[newJob]['tailFiles']["#STDERR#"]['path'] = os.path.join(jobWorkDirectory,
-                                                                                                            stdFile)
-                                    self.activeJobs[newJob]['tailFiles']["#STDERR#"]['endPosition'] = 0
-                           for tailFile in self.activeJobs[newJob]['tailFiles']:
-                              if tailFile != "#STDOUT#" and tailFile != "#STDERR#":
-                                 self.activeJobs[newJob]['tailFiles'][tailFile]['path'] = os.path.join(jobWorkDirectory,
-                                                                                                       tailFile)
-                                 self.activeJobs[newJob]['tailFiles'][tailFile]['endPosition'] = 0
-                        self.recordHistory(newJob)
-                        self.activeJobs[newJob]['status'] = 'n'
+                     self.activeJobs[newJob] = {'jobId':newJob,
+                                                'status':'N',
+                                                'stage':'Job',
+                                                'queue':'?',
+                                                'jobWorkDirectory':centralMessage.get('jobWorkDirectory','?'),
+                                                'localJobId':centralMessage.get('localJobId','?'),
+                                                'instanceId':centralMessage.get('instanceId','?'),
+                                                'runName':centralMessage.get('runName','?'),
+                                                'tailFiles':centralMessage.get('tailFiles',[])}
+                     jobWorkDirectory = os.path.expandvars(os.path.expanduser(self.activeJobs[newJob]['jobWorkDirectory']))
+                     if 'tailFiles' in self.activeJobs[newJob]:
+                        if 'runName' in self.activeJobs[newJob]:
+                           runName = self.activeJobs[newJob]['runName']
+                           if 'instanceId' in self.activeJobs[newJob]:
+                              instanceId = self.activeJobs[newJob]['instanceId']
+                              if "#STDOUT#" in self.activeJobs[newJob]['tailFiles']:
+                                 stdFile = "%s_%s.stdout" % (runName,instanceId)
+                                 self.activeJobs[newJob]['tailFiles']["#STDOUT#"]['path'] = os.path.join(jobWorkDirectory,
+                                                                                                         stdFile)
+                                 self.activeJobs[newJob]['tailFiles']["#STDOUT#"]['endPosition'] = 0
+                              if "#STDERR#" in self.activeJobs[newJob]['tailFiles']:
+                                 stdFile = "%s_%s.stderr" % (runName,instanceId)
+                                 self.activeJobs[newJob]['tailFiles']["#STDERR#"]['path'] = os.path.join(jobWorkDirectory,
+                                                                                                         stdFile)
+                                 self.activeJobs[newJob]['tailFiles']["#STDERR#"]['endPosition'] = 0
+                        for tailFile in self.activeJobs[newJob]['tailFiles']:
+                           if tailFile != "#STDOUT#" and tailFile != "#STDERR#":
+                              self.activeJobs[newJob]['tailFiles'][tailFile]['path'] = os.path.join(jobWorkDirectory,
+                                                                                                    tailFile)
+                              self.activeJobs[newJob]['tailFiles'][tailFile]['endPosition'] = 0
+                     self.recordHistory(newJob)
+                     self.activeJobs[newJob]['status'] = 'n'
                      consecutiveEmptyQueues = 0
                else:
                   self.cleanup()
@@ -483,7 +488,6 @@ class QueueMonitor:
             for job in jobs:
                jobId = job['jobId']
                currentJobs[jobId] = job
-#              currentJobs[jobId] = (status,stage)
 
             if len(currentJobs) == 0:
                consecutiveEmptyQueues += 1
@@ -502,21 +506,29 @@ class QueueMonitor:
                      completedJobs.append(activeJob)
 
             for currentJob in currentJobs:
-               if   not currentJob in self.activeJobs:
+               if not currentJob in self.activeJobs:
+                  log("recordHistory: currentJob not in activeJobs")
                   self.activeJobs[currentJob] = currentJobs[currentJob]
                   self.recordHistory(currentJob)
-               elif currentJobs[currentJob] != self.activeJobs[currentJob]:
+               else:
+                  somethingChanged = False
                   for key in currentJobs[currentJob]:
-                     self.activeJobs[currentJob][key] = currentJobs[currentJob][key]
-                  self.recordHistory(currentJob)
-               elif 'tailFiles' in self.activeJobs[currentJob]:
-                  for tailFile in self.activeJobs[currentJob]['tailFiles']:
-                     tailPath = self.activeJobs[currentJob]['tailFiles'][tailFile]['path']
-                     if os.path.exists(tailPath):
-                        fileSize = os.stat(tailPath).st_size
-                        if fileSize > self.activeJobs[currentJob]['tailFiles'][tailFile]['endPosition']:
-                           self.recordHistory(currentJob)
-                           break
+                     if currentJobs[currentJob][key] != self.activeJobs[currentJob][key]:
+                        somethingChanged = True
+                  if somethingChanged:
+                     log("recordHistory: currentJob different than activeJob")
+                     for key in currentJobs[currentJob]:
+                        self.activeJobs[currentJob][key] = currentJobs[currentJob][key]
+                     self.recordHistory(currentJob)
+                  elif 'tailFiles' in self.activeJobs[currentJob]:
+                     for tailFile in self.activeJobs[currentJob]['tailFiles']:
+                        tailPath = self.activeJobs[currentJob]['tailFiles'][tailFile]['path']
+                        if os.path.exists(tailPath):
+                           fileSize = os.stat(tailPath).st_size
+                           if fileSize > self.activeJobs[currentJob]['tailFiles'][tailFile]['endPosition']:
+                              log("recordHistory: change in tailFiles")
+                              self.recordHistory(currentJob)
+                              break
 
                if self.activeJobs[currentJob]['status'] == 'D':
                   completedJobs.append(currentJob)
@@ -528,15 +540,14 @@ class QueueMonitor:
             del completedJobs
 
             if len(self.updates) > 0:
-#              updateMessage = str(len(self.updates)) + ' ' + self.siteDesignator + ':' + ':'.join(self.updates)
-#              sys.stdout.write("%s\n" % (updateMessage))
-#              sys.stdout.flush()
                siteMessage = {'messageType':'siteUpdate','siteDesignator':self.siteDesignator,
                               'nJobStates':len(self.updates),'jobStates':self.updates}
                try:
+#                 log("WRITE: " + str(siteMessage))
                   sys.stdout.write(json.dumps(siteMessage) + '\n')
                except:
                   log("Site update failed")
+                  log(traceback.format_exc())
                else:
                   sys.stdout.flush()
 
@@ -566,6 +577,7 @@ if __name__ == '__main__':
       else:
          if sys.argv[1] != SITEDESIGNATOR:
             log("Site designators do not match.\n   internal name = %s\n   external name = %s" % (SITEDESIGNATOR,sys.argv[1]))
+            siteDesignator = ""
          else:
             siteDesignator = sys.argv[1]
    else:
